@@ -282,7 +282,6 @@ class VariableClUpdateOutdateSet(ReadonlyVariable):
     Если обновляются прочие оверлеи - данные считаются что устарели
     """
     type = "bool"
-    value = "off"
 
     def get(self):
         return self.Get('cl_update_other_set')
@@ -1042,6 +1041,7 @@ class VariableClUpdateKernelSrcPath(ReadonlyVariable):
         if path.exists(module_build_path):
             return os.readlink(module_build_path)
 
+
 class VariableClUpdateKernelPkg(ReadonlyVariable):
     """
     Пакет текущего ядра
@@ -1054,6 +1054,7 @@ class VariableClUpdateKernelPkg(ReadonlyVariable):
         else:
             return ""
 
+
 class VariableClUpdateLinesLimit(Variable):
     """
     Количество выводимых строк при ошибке
@@ -1061,6 +1062,7 @@ class VariableClUpdateLinesLimit(Variable):
     type = "int"
     value = "30"
 
+
 class VariableClUpdateSkipSetupSet(Variable):
     """
     Пропустить выполнение cl-setup-system в cl-update-profile
@@ -1072,3 +1074,16 @@ class VariableClUpdateSkipSetupSet(Variable):
     def init(self):
         self.label = _("Skip the system setup")
         self.help = _("skip the system setup")
+
+
+class VariableClUpdateCleanpkgSet(Variable):
+    """
+    Пропустить выполнение cl-setup-system в cl-update-profile
+    """
+    type = "bool"
+    value = "off"
+    opt = ["--clean-pkg"]
+
+    def init(self):
+        self.label = _("Clean obsolete programs archives")
+        self.help = _("clean obsolete programs archives")
