@@ -69,6 +69,7 @@ class Wsdl(WsdlBase):
                                     'cl_update_other_set',
                                     'cl_update_pretend_set',
                                     'cl_update_sync_rep',
+                                    'cl_update_cleanpkg_set',
                                     'cl_update_emergelist_set',
                                     'cl_update_world',
                                     'cl_update_egencache_force',
@@ -164,6 +165,8 @@ class Wsdl(WsdlBase):
             'groups': [
                 lambda group: group(_("Updates autocheck settings"),
                                     normal=('cl_update_autocheck_set',
-                                            'cl_update_autocheck_interval'),
+                                            'cl_update_autocheck_interval',
+                                            'cl_update_cleanpkg_set',
+                                            'cl_update_other_set'),
                                     next_label=_("Save"))]},
     ]
